
# q-expansion of newform 3825.2.a.be, downloaded from the LMFDB on 22 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3825
weight = 2
poly_data = [-3, -6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-4, -1, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, -1, 0], [0, 0, 0], [0, 0, 0], [0, 0, -1], [-2, 0, 1], [-2, 0, 0], [-1, 0, 0], [4, 0, -1], [0, 0, 0], [-4, -2, -1], [0, 0, -2], [2, 2, 1], [0, 2, 3], [-2, 4, 0], [0, 4, 3], [-4, 2, -1], [4, 0, -2], [-2, 4, 2], [-6, -4, 2], [-2, -4, -2], [-6, 2, -1], [8, 0, 0], [-4, 0, -4], [-4, -4, 2], [-4, -4, -2], [4, 0, 4], [-6, 4, 2], [-8, 0, -2], [-2, -4, 2], [2, 4, 2], [-6, -4, 2], [-2, 4, -2], [-4, 2, 5], [4, 0, 2], [4, 4, -2], [-4, 4, 3], [6, -8, -4], [0, -4, 5], [12, 0, 0], [-6, -4, 6], [0, -8, 0], [-2, -4, 2], [-8, 8, 4], [0, 4, 2], [2, 0, -4], [0, -8, -2], [20, 0, 0], [-6, -4, -10], [-4, 0, 8], [4, 2, -1], [-10, -4, 2], [-4, 8, 2], [6, 4, -2], [-8, 0, -2], [2, -8, -4], [-8, 4, 1], [0, 2, -3], [-4, -8, 0], [0, 4, 2], [8, 0, -4], [-12, 4, -1], [-8, -6, 7], [-10, 4, -2], [-10, 4, 5], [6, 2, 11], [-6, 8, 0], [0, 0, 7], [10, -6, 3], [4, -8, -8], [-4, -6, 3], [-8, 10, 1], [4, 0, -2], [-16, -8, 4], [14, 4, -2], [12, -8, -2], [-24, -4, -3], [-4, -4, -10], [-6, 2, 5], [4, 6, 1], [0, -14, 1], [6, 0, -9], [-8, -2, -7], [10, 4, -5], [10, -4, 6], [-8, 0, -4], [4, 0, -8], [-10, 8, -4], [6, -4, 2], [12, -8, 0], [-2, -4, -6], [-8, -8, 1], [-18, -4, -6], [8, -8, -8], [12, 0, -6], [12, -16, -2], [-12, -8, -6], [-28, 0, -4], [4, 6, 1], [6, -4, -4], [-14, 8, -4], [-16, -4, -5], [-14, 0, -8], [-8, 0, 7], [-8, -4, -2], [12, 8, 10], [10, 8, 0], [-16, 8, 5], [12, 2, 9], [-20, -8, 4], [-26, -8, 8], [20, 0, -5], [30, 0, -4], [22, -4, -2], [4, 8, 8], [-12, 4, -5], [40, -2, 1], [-24, -4, -1], [0, 0, -12], [-14, 4, -2], [-4, 8, 2], [-12, -6, -5], [32, -4, -2], [26, -16, -4], [-4, 0, 2], [4, 8, -10], [-24, 8, 0], [10, 12, -10], [-26, 4, -5], [6, 4, 2], [-10, -8, -8], [-12, 16, 4], [-8, 0, 4], [-32, -8, 2], [26, 8, 4], [32, -4, 2], [0, -2, -5], [16, 14, 1], [0, 4, 11], [-4, -2, -7], [6, -16, 0], [4, 16, 8], [-6, -8, -12], [24, 8, 17], [8, 8, -4], [-12, -10, 1], [2, -12, -1], [16, 12, 6], [26, 8, 8], [4, 0, 5], [-16, 4, 5], [-14, -6, -3], [16, -22, -5], [-10, 4, -8], [0, -8, 0], [-20, -4, -9], [-10, 4, -7], [-24, 4, -5], [-4, 10, 11], [-10, -12, -2], [14, 0, 8], [-12, 0, -12], [-20, -14, 1], [30, -4, -4], [28, 8, 4], [14, 8, 20], [8, 0, -10], [-24, -8, -8], [10, -18, 3], [-14, 0, 8], [-10, -4, 2], [-4, -8, 2], [14, -16, 0], [-26, -4, -14], [-38, 4, -6], [-20, 8, 8], [-6, 16, 0], [4, 0, -4], [14, 0, 8], [10, -12, -6], [18, 0, -8], [-18, 20, 8], [-34, -8, -7], [6, -2, -13], [12, -6, -3], [-16, 0, -4], [-24, 0, 0], [30, 2, 5], [-18, 20, 2], [30, 0, -8], [-6, -20, 9], [-6, -2, 5], [4, 0, 10], [20, -16, -3], [0, 12, 6], [-12, -16, 12], [18, 4, -6], [-2, 16, 8], [-10, 10, 13], [-26, 4, -2], [8, -16, -4], [4, 22, -5], [16, 8, -4], [-42, 12, 2], [0, 6, 7], [12, -16, -12], [0, -6, 3], [24, 0, 4], [-12, 0, -12], [-48, 10, 3], [32, 0, -3], [-10, 22, 1], [32, 12, -10], [4, 0, 3], [-16, 0, -1], [-10, 4, 5], [-2, 0, -4], [10, 20, -6], [12, 12, -6], [-12, -16, -6], [-22, 0, 8], [-22, -8, 12], [-8, 8, 14], [20, -4, 14], [16, -24, -12], [40, 0, -5], [8, 6, 9], [2, 8, -16], [-30, -12, -6], [-2, 12, 12], [-18, -8, -15], [2, -12, -2], [36, 16, -8], [0, 20, 7], [12, -8, 12], [-14, 4, 12], [-16, 16, -4], [6, 16, 16], [14, 4, 14], [-38, 8, 1], [-8, 8, 16], [36, 0, 6], [-24, -8, 7], [18, 20, 2], [6, 16, -8], [-2, 0, -8], [22, 20, -11], [16, 8, 15], [14, -8, 11], [-8, 0, 11], [-24, -12, -9], [-6, 8, -4], [-4, 22, 17], [-32, 12, -5], [10, -4, -10], [-48, -10, -3], [64, -8, -2], [-6, 0, -8], [-12, -4, -19], [10, 16, -8], [-30, 20, 2], [28, 0, 0], [-32, -8, 1], [18, 12, -14], [26, -18, -5], [46, -4, 10], [0, -8, -8], [4, 4, -14], [-40, 0, -4], [-8, 0, 0], [16, -18, 1], [6, 16, 4], [-14, 20, 12], [10, -12, -6], [-48, 4, 1], [-26, 0, 12], [-18, 12, 8], [-40, -8, -1], [8, 10, -21], [12, -18, -5], [-52, 8, 8], [0, 0, 12], [0, -8, -14], [-36, -8, -6], [-6, -8, 4], [-36, 8, -4], [-16, 24, 2], [18, -22, -13], [18, 4, -6], [-8, -20, -7], [16, -8, 16], [12, -14, -15], [48, 8, 6], [40, -14, -11], [-22, 16, 5], [-22, 10, -11], [-32, -10, 7], [0, 4, 19], [4, 6, 1], [22, 0, 7], [26, -4, 4], [-34, 4, 10], [16, -2, -11], [-32, 16, 8], [-4, 24, 20], [4, -8, 8], [18, -10, -19], [-40, 16, -1], [42, -12, -2], [-20, -16, -14], [-26, -16, 12], [4, 0, 10], [44, 14, -1], [-6, -8, -12], [50, 4, 16], [-24, 16, 12], [24, -30, -11], [52, 8, 4], [-22, -12, 11], [24, 12, 14], [-8, -8, 4], [12, 0, -14], [-36, -4, 14], [0, -20, 6], [14, -12, 4], [-2, 28, 10], [0, -14, 13], [-12, -8, 4], [-12, -4, -1], [-44, 0, -8], [34, 8, 4], [34, -4, 2], [-2, 12, -2], [-24, -8, -20], [24, -24, -3], [-4, -16, -12], [-4, 24, -10], [-16, 22, 9], [-22, 28, 2], [10, 0, -4], [-2, 12, 0], [30, 20, -10], [6, 16, 0], [8, -10, -13], [12, -8, -8], [50, -8, -4], [46, -24, -5], [-34, 0, 0], [-14, 4, 18], [-26, -12, -5], [26, 4, -10], [16, 8, -16], [10, -22, -21], [16, 4, -2], [-36, 24, 11], [-14, 8, 20], [12, 14, -9], [52, -16, -8], [6, -16, 9], [18, -32, -12], [-48, -4, -9], [48, -4, -10], [6, 8, -12], [56, -4, -7], [30, 20, -6], [10, 12, -12], [-70, -8, -8], [-42, 4, 6], [10, -12, -12], [46, 8, 20], [-64, -8, 2], [28, 0, 2], [-28, 8, -10], [-32, -20, 22], [-36, -8, -8], [-8, 20, 18], [-2, -12, -14], [-38, 4, -11], [14, 16, 4], [0, -4, -30], [26, -16, -8], [-6, 0, 0], [28, 10, -11], [12, -8, 11], [-22, 16, 20], [-20, -8, -10], [-4, -8, -4], [-16, 4, 15], [-10, -12, 10], [16, 0, 24], [-56, 12, 1], [-34, -16, 12], [-14, -28, -14], [0, 16, 12], [-18, -28, -4], [-32, 16, 10], [32, 12, -2], [-8, 20, -1], [-28, -4, 2], [-28, -8, 6], [8, 8, 20], [58, -4, 2], [12, -14, 9], [2, -20, 10], [22, 0, 4], [-16, 16, 8], [-64, 24, 6], [-10, -12, -14], [4, 16, 28], [-38, -12, 12], [30, 4, -6], [22, 30, -3], [24, -6, 3], [-16, -8, 17], [-36, -8, -8], [-14, 16, 0], [0, -26, -21], [-42, -4, 18], [32, -24, 1], [66, -12, -6], [0, 16, 0], [28, 8, -20], [-6, -14, 5], [-12, -16, 0], [0, -24, 0], [-34, -14, -5], [-2, 0, 16], [28, 16, -14], [4, 30, 25], [-4, 16, 12], [2, -12, -22]]
